<?php


if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'Af_Wc_Daps_Admin' ) ) {

	class Af_Wc_Daps_Admin {

		public function __construct() {

			add_action( 'admin_enqueue_scripts', array( $this, 'af_wc_daps_enqueue_admin_scripts' ) );

			add_action( 'add_meta_boxes', array( $this, 'af_wc_daps_admin_add_metaboxes' ) );
			add_action( 'save_post_daps_rules', array( $this, 'af_wc_daps_save_metabox_content' ) );
			add_action( 'woocommerce_process_product_meta', array( $this, 'af_wc_daps_save_metabox_content' ) );

			add_filter( 'woocommerce_settings_tabs_array', array( $this, 'af_wc_daps_settings_tab' ) , 50 );
			add_action( 'woocommerce_settings_tabs_settings_tab_order_delivery', array( $this, 'settings_tab' ) );
			add_action( 'woocommerce_settings_save_settings_tab_order_delivery', array( $this, 'save_settings' ) );

			add_action( 'save_post_daps_holidays', array( $this, 'af_wc_daps_save_holidays_content' ) );
			add_action( 'save_post_daps_special_days', array( $this, 'af_wc_daps_save_special_days_content' ) );

			add_action( 'wp_ajax_af_wc_daps_product_search', array( $this, 'af_wc_daps_product_search' ) );

			add_filter( 'manage_daps_order_delivery_posts_columns', array( $this, 'af_wc_daps_order_delivery_table_columns' ) );
			add_action( 'manage_daps_order_delivery_posts_custom_column', array( $this, 'af_wc_daps_order_delivery_table_column' ), 10, 2 );

			add_filter( 'manage_daps_holidays_posts_columns', array( $this, 'af_wc_daps_holidays_table_columns' ) );
			add_action( 'manage_daps_holidays_posts_custom_column', array( $this, 'af_wc_daps_holidays_table_column' ), 10, 2 );

			add_filter( 'manage_daps_special_days_posts_columns', array( $this, 'af_wc_daps_special_days_table_columns' ) );
			add_action( 'manage_daps_special_days_posts_custom_column', array( $this, 'af_wc_daps_special_days_table_column' ), 10, 2 );

			add_action( 'all_admin_notices', array( $this, 'af_wc_daps_display_tabs' ), 5 );
			add_action( 'admin_menu', array( $this, 'af_wc_daps_remove_submenu_link' ) );


			add_filter( 'manage_edit-daps_order_delivery_sortable_columns', array( $this, 'af_wc_daps_unset_sortable_column' ) );
			add_filter( 'manage_edit-daps_prod_delivery_sortable_columns', array( $this, 'af_wc_daps_unset_sortable_column' ) );

			add_filter ('bulk_actions-edit-daps_order_delivery' , array( $this, 'af_wc_daps_remove_bulk_edit' ));
			add_filter ('bulk_actions-edit-daps_holidays' , array( $this, 'af_wc_daps_remove_bulk_edit' ));
			add_filter ('bulk_actions-edit-daps_special_days' , array( $this, 'af_wc_daps_remove_bulk_edit' ));

			add_filter( 'post_updated_messages', array( $this, 'af_wc_daps_remove_view_post_link_message' ), 999 );
			add_filter( 'post_row_actions', array( $this, 'af_wc_daps_remove_quick_edit_from_custom_post_table' ), 10, 2 );

			add_action( 'woocommerce_admin_order_totals_after_total', array( $this, 'af_wc_daps_display_delivery_info_data_at_edit_order_page' ), 10, 1 );
			add_action( 'woocommerce_after_order_itemmeta', array( $this, 'af_wc_daps_display_delivery_info_for_individual_product' ) , 10, 3);
		}



		public function af_wc_daps_settings_tab( $settings_tabs ) {
			$settings_tabs['settings_tab_order_delivery'] = esc_html__( 'Order Delivery', 'ka-order-delivery-date-and-time-scheduler' );
			return $settings_tabs;
		}

		public function settings_tab() {
			
			$current_section = filter_input(INPUT_GET, 'section', FILTER_SANITIZE_FULL_SPECIAL_CHARS);

			if (  '' == $current_section  ) {
				$current_section = 'general';
			} 
			?>

			<ul class="subsubsub">
				<li><a href="?page=wc-settings&tab=settings_tab_order_delivery&section=general" class="<?php echo ( 'general' === $current_section ) ? 'current' : ''; ?>"><?php esc_html_e( 'General', 'ka-order-delivery-date-and-time-scheduler' ); ?></a> |</li>
				<li><a href="?page=wc-settings&tab=settings_tab_order_delivery&section=calendar_style" class="<?php echo ( 'calendar_style' === $current_section ) ? 'current' : ''; ?>"><?php esc_html_e( 'Calendar Style', 'ka-order-delivery-date-and-time-scheduler' ); ?></a> |</li>
				<li><a href="?page=wc-settings&tab=settings_tab_order_delivery&section=expected_info_style" class="<?php echo ( 'expected_info_style' === $current_section ) ? 'current' : ''; ?>"><?php esc_html_e( 'Expected Date Style', 'ka-order-delivery-date-and-time-scheduler' ); ?></a> |</li>
				<li><a href="?page=wc-settings&tab=settings_tab_order_delivery&section=timezone" class="<?php echo ( 'timezone' === $current_section ) ? 'current' : ''; ?>"><?php esc_html_e( 'Time Zone', 'ka-order-delivery-date-and-time-scheduler' ); ?></a> |</li>
				<li><a href="?page=wc-settings&tab=settings_tab_order_delivery&section=label" class="<?php echo ( 'label' === $current_section ) ? 'current' : ''; ?>"><?php esc_html_e( 'Label', 'ka-order-delivery-date-and-time-scheduler' ); ?></a> </li>



			</ul><br><br>
			<?php

			include_once AF_WC_DAPS_PLUGIN_DIR . '/includes/admin/settings/af-wc-general-setting.php';
			include_once AF_WC_DAPS_PLUGIN_DIR . '/includes/admin/settings/af-wc-label-setting.php';
			include_once AF_WC_DAPS_PLUGIN_DIR . '/includes/admin/settings/af-wc-calendar-style-setting.php';
			include_once AF_WC_DAPS_PLUGIN_DIR . '/includes/admin/settings/af-wc-expected-info-style-setting.php';
			include_once AF_WC_DAPS_PLUGIN_DIR . '/includes/admin/settings/af-wc-timezone-setting.php';

			wp_nonce_field( 'af_wc_daps_nonce', 'af_wc_daps_nonce_field' );


			if ( 'general' === $current_section ) {
				settings_fields( 'af-wc-daps-general-settings-group' );
				do_settings_sections( 'af-wc-daps-general-settings' );
			} elseif ( 'calendar_style' === $current_section ) {
				settings_fields( 'af-wc-daps-calendar-style-settings-group' );
				do_settings_sections( 'af-wc-daps-calendar-style-settings' );
			} elseif ( 'expected_info_style' === $current_section ) {
				settings_fields( 'af-wc-daps-expected-info-style-settings-group' );
				do_settings_sections( 'af-wc-daps-expected-info-style-settings' );
			} elseif ( 'timezone' === $current_section ) {
				settings_fields( 'af-wc-daps-timezone-settings-group' );
				do_settings_sections( 'af-wc-daps-timezone-settings' );
				do_settings_sections( 'wpo_noti_section' );
			} elseif ( 'label' === $current_section ) {
				settings_fields( 'af-wc-daps-label-settings-group' );
				do_settings_sections( 'af-wc-daps-label-settings' );
			}
		}

		public function save_settings() {
			
			$current_section = filter_input(INPUT_GET, 'section', FILTER_SANITIZE_FULL_SPECIAL_CHARS);

			if (  '' == $current_section  ) {
				$current_section = 'general';
			} 
			include_once AF_WC_DAPS_PLUGIN_DIR . '/includes/admin/settings/af-wc-save-setting.php';
		}
	
		

		

		public function af_wc_daps_remove_bulk_edit( $actions ) {
			unset($actions['edit']);
			return $actions;
		}

		public function af_wc_daps_remove_view_post_link_message( $messages ) {

			global $post;

			if ( $post && ( 'daps_rules' == $post->post_type || 'daps_special_days' == $post->post_type || 'daps_holidays' == $post->post_type )) {
				$messages['post'][1] = __( 'Post Updated.', 'ka-order-delivery-date-and-time-scheduler' );
				$messages['post'][6] = __( 'Post published.', 'ka-order-delivery-date-and-time-scheduler' );
			}

			return $messages;
		}

		public function af_wc_daps_unset_sortable_column( $sortable_columns ) {
			unset($sortable_columns['date']);
			return $sortable_columns;
		}

		public function af_wc_daps_admin_add_metaboxes() {
			$this->af_wc_daps_add_metaboxes();
		}

		public function af_wc_daps_order_delivery_table_columns( $columns ) {
			unset( $columns['date'] );
			unset( $columns['title'] );
			$columns['af_wc_daps_order_id']         = esc_html__( 'Order Id', 'ka-order-delivery-date-and-time-scheduler' );
			$columns['af_wc_daps_user_details']     = esc_html__( 'User Details', 'ka-order-delivery-date-and-time-scheduler' );
			$columns['af_wc_daps_products']         = esc_html__( 'Products', 'ka-order-delivery-date-and-time-scheduler' );
			$columns['af_wc_daps_selection_method'] = esc_html__( 'Selection Method', 'ka-order-delivery-date-and-time-scheduler' );
			$columns['af_wc_daps_delivery_date']    = esc_html__( 'Delivery Date', 'ka-order-delivery-date-and-time-scheduler' );
			$columns['af_wc_daps_time_slots']       = esc_html__( 'Time Slot / Time', 'ka-order-delivery-date-and-time-scheduler' );
			$columns['af_wc_daps_delivery_fee']     = esc_html__( 'Delivery Fee', 'ka-order-delivery-date-and-time-scheduler' );
			$columns['date']                        = esc_html__( 'Date', 'ka-order-delivery-date-and-time-scheduler' );
			return $columns;
		}

		public function af_wc_daps_order_delivery_table_column( $column, $post_id ) {

			$af_wc_daps_order_table_settings = get_post_meta( $post_id, 'af_wc_daps_order_delivery_info', true );

			switch ( $column ) {
				case 'af_wc_daps_order_id':
					$af_wc_daps_order_id = isset( $af_wc_daps_order_table_settings['af_wc_daps_order_id'] ) ? esc_html( $af_wc_daps_order_table_settings['af_wc_daps_order_id'] ) : '';
					$order               = wc_get_order($af_wc_daps_order_id);
					$order_edit_link     =null;
					if ($order) {
						$order_edit_link = $order->get_edit_order_url() ;
					}
					if ( null == $order_edit_link ) {
						echo esc_attr($af_wc_daps_order_id);
					} else {
						
						echo '<a target="_blank" href="' . esc_url( $order_edit_link ) . '"><i class="fa fa-eye"></i> ' . esc_attr( $af_wc_daps_order_id ) . '</a>';
					}
					break;
				case 'af_wc_daps_user_details':
					$af_wc_daps_user_details = isset( $af_wc_daps_order_table_settings['af_wc_daps_user_details'] ) ? esc_html( $af_wc_daps_order_table_settings['af_wc_daps_user_details'] ) : '';
					echo esc_attr($af_wc_daps_user_details);
					break;
				case 'af_wc_daps_products':
					$af_wc_daps_products = isset( $af_wc_daps_order_table_settings['af_wc_daps_products'] ) ? $af_wc_daps_order_table_settings['af_wc_daps_products'] : array();
					foreach ( $af_wc_daps_products as $product ) {
						$product_id = esc_attr($product['af_wc_daps_product_id']);

						$product_edit_link = get_edit_post_link( $product_id );

						if ( $product_edit_link ) {
							echo '<a href="' . esc_url( $product_edit_link ) . '" target="_blank">' . esc_attr($product['af_wc_daps_product']) . '</a>, ';
						} else {
							echo esc_attr($product['af_wc_daps_product']) . ', ';
						}
					}
					break;
				case 'af_wc_daps_selection_method':
					$af_wc_daps_selection_method = isset( $af_wc_daps_order_table_settings['af_wc_daps_selection_method'] ) ? esc_html( $af_wc_daps_order_table_settings['af_wc_daps_selection_method'] ) : '';
					echo esc_attr($af_wc_daps_selection_method);
					break;

				case 'af_wc_daps_delivery_date':
					if ( isset( $af_wc_daps_order_table_settings['af_wc_daps_selection_method'] ) && 'Expected Date' == $af_wc_daps_order_table_settings['af_wc_daps_selection_method'] ) {
						$af_wc_daps_expected_date_info = isset( $af_wc_daps_order_table_settings['af_wc_daps_expected_date_info'] ) ? esc_html( $af_wc_daps_order_table_settings['af_wc_daps_expected_date_info'] ) : '';
						echo esc_attr($af_wc_daps_expected_date_info);
						break;
					} elseif ( isset( $af_wc_daps_order_table_settings['af_wc_daps_selection_method'] ) && 'Calendar' == $af_wc_daps_order_table_settings['af_wc_daps_selection_method'] ) {
						$af_wc_daps_delivery_date = isset( $af_wc_daps_order_table_settings['af_wc_daps_delivery_date'] ) ? esc_html( $af_wc_daps_order_table_settings['af_wc_daps_delivery_date'] ) : '';
						echo esc_attr($af_wc_daps_delivery_date);
						break;
					}
					break;

				case 'af_wc_daps_time_slots':
					$af_wc_daps_time_slots = isset( $af_wc_daps_order_table_settings['af_wc_daps_time_slots'] ) ? esc_html( $af_wc_daps_order_table_settings['af_wc_daps_time_slots'] ) : '';
					echo esc_attr($af_wc_daps_time_slots);
					break;
				case 'af_wc_daps_delivery_fee':
					$af_wc_daps_delivery_fee = isset( $af_wc_daps_order_table_settings['af_wc_daps_delivery_fee'] ) ? esc_html( $af_wc_daps_order_table_settings['af_wc_daps_delivery_fee'] ) : '';
					echo esc_attr(get_woocommerce_currency_symbol()) . esc_attr($af_wc_daps_delivery_fee);
					break;
				default:
					break;
			}
		}

		public function af_wc_daps_remove_quick_edit_from_custom_post_table( $actions, $post ) {
			if ( 'daps_order_delivery' == $post->post_type ) {
				unset( $actions['edit'] );
				unset( $actions['view'] );
				unset( $actions['inline hide-if-no-js'] );
				return $actions;
			} elseif ('daps_holidays' == $post->post_type || 'daps_special_days' == $post->post_type) {
				unset( $actions['view'] );
				unset( $actions['inline hide-if-no-js'] );
				return $actions;
			} elseif ('daps_rules' === $post->post_type) {
				unset( $actions['view'] );
				unset( $actions['inline hide-if-no-js'] );
				return $actions;
			}

			return $actions;
		}

		public function af_wc_daps_holidays_table_columns( $columns ) {
			unset( $columns['date'] );
			$columns['af_wc_daps_holiday_start_date']       = esc_html__( 'Start Date', 'ka-order-delivery-date-and-time-scheduler' );
			$columns['af_wc_daps_holiday_end_date']         = esc_html__( 'End Date', 'ka-order-delivery-date-and-time-scheduler' );
			$columns['af_wc_daps_holiday_recurring_status'] = esc_html__( 'Is Recurring?', 'ka-order-delivery-date-and-time-scheduler' );
			$columns['date']                                = esc_html__( 'Date', 'ka-order-delivery-date-and-time-scheduler' );
			return $columns;
		}

		public function af_wc_daps_holidays_table_column( $column, $post_id ) {

			$af_wc_daps_holiday_settings = get_post_meta( $post_id, 'af_wc_daps_holiday_settings', true );

			switch ( $column ) {
				case 'af_wc_daps_holiday_start_date':
					$af_wc_daps_holiday_start_date = isset( $af_wc_daps_holiday_settings['af_wc_daps_holiday_start_date'] ) ? esc_html( $af_wc_daps_holiday_settings['af_wc_daps_holiday_start_date'] ) : '';
					echo esc_attr($af_wc_daps_holiday_start_date);
					break;
				case 'af_wc_daps_holiday_end_date':
					$af_wc_daps_holiday_end_date = isset( $af_wc_daps_holiday_settings['af_wc_daps_holiday_end_date'] ) ? esc_html( $af_wc_daps_holiday_settings['af_wc_daps_holiday_end_date'] ) : '';
					echo esc_attr($af_wc_daps_holiday_end_date);
					break;
				case 'af_wc_daps_holiday_recurring_status':
					$af_wc_daps_holiday_recurring_status = isset( $af_wc_daps_holiday_settings['af_wc_daps_repeat_holiday'] ) && '' !=$af_wc_daps_holiday_settings['af_wc_daps_repeat_holiday'] ? esc_html( $af_wc_daps_holiday_settings['af_wc_daps_repeat_holiday'] ) : 'no';
					echo esc_attr($af_wc_daps_holiday_recurring_status);
					break;
				default:
					break;
			}
		}

		public function af_wc_daps_special_days_table_columns( $columns ) {
			unset( $columns['date'] );
			$columns['af_wc_daps_specific_day_date'] = esc_html__( 'Special Day Date', 'ka-order-delivery-date-and-time-scheduler' );
			$columns['af_wc_daps_maximum_orders']    = esc_html__( 'Maximum Orders', 'ka-order-delivery-date-and-time-scheduler' );
			$columns['af_wc_daps_remaining_orders']  = esc_html__( 'Remaining Orders', 'ka-order-delivery-date-and-time-scheduler' );
			$columns['af_wc_daps_charges']           = esc_html__( 'Charges', 'ka-order-delivery-date-and-time-scheduler' );
			$columns['date']                         = esc_html__( 'Date', 'ka-order-delivery-date-and-time-scheduler' );
			return $columns;
		}

		public function af_wc_daps_special_days_table_column( $column, $post_id ) {

			$af_wc_daps_specific_day_settings = get_post_meta( $post_id, 'af_wc_daps_specific_day_settings', true );
			$af_wc_daps_remaining_orders      = get_post_meta( $post_id, 'af_wc_daps_remaining_orders_of_specific_day', true );

			switch ( $column ) {
				case 'af_wc_daps_specific_day_date':
					$af_wc_daps_specific_day_date = isset( $af_wc_daps_specific_day_settings['af_wc_daps_specific_day_date'] ) ? esc_html( $af_wc_daps_specific_day_settings['af_wc_daps_specific_day_date'] ) : '';
					echo esc_attr($af_wc_daps_specific_day_date);
					break;
				case 'af_wc_daps_charges':
					$af_wc_daps_specific_day_charges = isset( $af_wc_daps_specific_day_settings['af_wc_daps_specific_day_charges'] ) ? esc_html( $af_wc_daps_specific_day_settings['af_wc_daps_specific_day_charges'] ) : '';
					if ('' != $af_wc_daps_specific_day_charges) {
						echo esc_attr(get_woocommerce_currency_symbol()) . esc_attr($af_wc_daps_specific_day_charges);
					}
					break;
				case 'af_wc_daps_maximum_orders':
					$af_wc_daps_specific_day_max_orders = isset( $af_wc_daps_specific_day_settings['af_wc_daps_specific_day_max_orders'] ) ? esc_html( $af_wc_daps_specific_day_settings['af_wc_daps_specific_day_max_orders'] ) : '';
					echo esc_attr($af_wc_daps_specific_day_max_orders);
					break;
				case 'af_wc_daps_remaining_orders':
					$af_wc_daps_specific_day_max_orders = isset( $af_wc_daps_remaining_orders ) ? esc_html( $af_wc_daps_remaining_orders ) : '';
					echo esc_attr($af_wc_daps_specific_day_max_orders);
					break;
				default:
					break;
			}
		}

		public function af_wc_daps_enqueue_admin_scripts() {

			$af_wc_daps_current_screen = get_current_screen();          
			
			if ( $af_wc_daps_current_screen && ( in_array($af_wc_daps_current_screen->id, $this-> get_screen_tab_id() ) )) {
								
				wp_enqueue_script( 'jquery' );
				wp_enqueue_script( 'jquery-ui-datepicker' );
				wp_enqueue_script( 'jquery-ui-core' );
				
				wp_enqueue_style( 'af_wc_daps-fontawesome', AF_WC_DAPS_PLUGIN_URL . '/assets/external-dependencies/fontawesome-free-6.6.0-web/css/all.min.css', array(), '6.6.0' );
				wp_enqueue_style( 'jquery-ui-timepicker-addon', AF_WC_DAPS_PLUGIN_URL . '/assets/external-dependencies/jquery-ui-timepicker-addon-1.6.3/jquery-ui-timepicker-addon.css', array(), '1.6.3' );
				wp_enqueue_script( 'jquery-ui-timepicker-addon', AF_WC_DAPS_PLUGIN_URL . '/assets/external-dependencies/jquery-ui-timepicker-addon-1.6.3/jquery-ui-timepicker-addon.js', array( 'jquery', 'jquery-ui-core', 'jquery-ui-datepicker' ), '1.6.3', true );
				
			
				$af_wc_daps_calendar_style_settings = (array) get_option( 'af_wc_daps_calendar_style_settings' );
				$af_wc_daps_admin_calendar_theme    = isset( $af_wc_daps_calendar_style_settings['af_wc_daps_admin_calendar_theme'] ) ? $af_wc_daps_calendar_style_settings['af_wc_daps_admin_calendar_theme'] : 'base';
				
				wp_enqueue_style( 'jquery-ui-theme', AF_WC_DAPS_PLUGIN_URL . '/assets/external-dependencies/jquery-ui-themes-1.14.1/themes/' . $af_wc_daps_admin_calendar_theme . '/jquery-ui.css', '1.14.1', '1.14.1' );

				wp_enqueue_style( 'select2-css', plugins_url( 'assets/css/select2.css', WC_PLUGIN_FILE ), array(), '5.7.2' );
				wp_enqueue_script( 'select2-js', plugins_url( 'assets/js/select2/select2.min.js', WC_PLUGIN_FILE ), array( 'jquery' ), '4.0.3', true );
				
				wp_enqueue_style( 'af-wc-daps-admin-style', AF_WC_DAPS_PLUGIN_URL . 'assets/css/af-wc-daps-admin-style.css', array(), '1.0' );
				wp_enqueue_script( 'af-wc-daps-admin-script', AF_WC_DAPS_PLUGIN_URL . 'assets/js/af-wc-daps-admin-script.js', array( 'jquery' ), '1.0.0', true );

				wp_enqueue_script( 'media-upload' );
				wp_enqueue_media();

				ob_start();
				?>
				<tr class="template-row">			
					<td class="af-wc-daps-column-start-time">
						<input  type="text" name="af_wc_daps_time_slot_settings[][start_time]"  class="af-wc-daps-time-slot-start-time-input" required> 
						<p class=description><?php esc_html_e('Add start time of time slot.', 'ka-order-delivery-date-and-time-scheduler'); ?></p>
					</td>

					<td class="af-wc-daps-column-end-time">
						<input  type="text" name="af_wc_daps_time_slot_settings[][end_time]"  class="af-wc-daps-time-slot-end-time-input" required>
						<p class=description><?php esc_html_e('Add end time of time slot.', 'ka-order-delivery-date-and-time-scheduler'); ?></p>
					</td>

					<td class="af-wc-daps-column-charges">
						<input  type="number" min="1" name="af_wc_daps_time_slot_settings[][charges]" class="af-wc-daps-time-slot-charges-input">
						<p class=description><?php esc_html_e('Add charges of time slot.', 'ka-order-delivery-date-and-time-scheduler'); ?></p>
					</td>

					<td class="af-wc-daps-column-days-of-week">
						<select class="af-wc-daps-days-of-week-select" name="af_wc_daps_time_slot_settings[][days_of_week][]" multiple="multiple" class="af-wc-daps-days-of-week-select">
							<option value="mon" ><?php esc_html_e( 'Monday', 'ka-order-delivery-date-and-time-scheduler' ); ?></option>
							<option value="tue" ><?php esc_html_e( 'Tuesday', 'ka-order-delivery-date-and-time-scheduler' ); ?></option>
							<option value="wed" ><?php esc_html_e( 'Wednesday', 'ka-order-delivery-date-and-time-scheduler' ); ?></option>
							<option value="thu" ><?php esc_html_e( 'Thursday', 'ka-order-delivery-date-and-time-scheduler' ); ?></option>
							<option value="fri" ><?php esc_html_e( 'Friday', 'ka-order-delivery-date-and-time-scheduler' ); ?></option>
							<option value="sat" ><?php esc_html_e( 'Saturday', 'ka-order-delivery-date-and-time-scheduler' ); ?></option>
							<option value="sun" ><?php esc_html_e( 'Sunday', 'ka-order-delivery-date-and-time-scheduler' ); ?></option>
						</select>
							<p class=description><?php esc_html_e('Choose days on which this time slot is applicable.Leave blank for default value (all days).', 'ka-order-delivery-date-and-time-scheduler'); ?></p>
							<input type="hidden" class="index" >
					</td>

					<td class="af-wc-daps-column-orders-per-slot">
						<input  type="number" min="1" name="af_wc_daps_time_slot_settings[][orders_per_slot]" class="af-wc-daps-orders-per-slot-input">
						<p class=description><?php esc_html_e('Add maximum number of orders allowed per slot.', 'ka-order-delivery-date-and-time-scheduler'); ?></p>
					</td>
										
					<td  class="af-wc-daps-column-delete"><i class="fa fa-trash af-wc-daps-delete-button"></i></td>
											
				</tr>

				<?php
				$af_wc_daps_timeslot_settings_template_row = ob_get_clean();

				wp_localize_script(
					'af-wc-daps-admin-script',
					'af_wc_daps_script_data',
					array(
						'ajax_url'         => admin_url( 'admin-ajax.php' ),
						'af_wc_daps_nonce' => wp_create_nonce( 'af-wc-daps-ajax-nonce' ),
						'af_wc_daps_timeslot_settings_template_row' => $af_wc_daps_timeslot_settings_template_row,

					)
				);

			}
		}

		public function get_screen_tab_id() {
			$tabs =array( 'daps_rules', 'product', 'woocommerce_page_daps-settings', 'edit-daps_order_delivery', 'edit-daps_order_delivery', 'edit-daps_prod_delivery', 'edit-daps_rules', 'daps_holidays', 'edit-daps_holidays', 'daps_special_days', 'edit-daps_special_days', 'woocommerce_page_wc-orders', 'woocommerce_page_wc-settings' );
			return $tabs;
		}

		public function af_wc_daps_add_metaboxes() {
			add_meta_box(
				'af_wc_daps_holidays_general_options',
				__( 'Holidays', 'ka-order-delivery-date-and-time-scheduler' ),
				array( $this, 'af_wc_daps_holidays_general_section_content' ),
				'daps_holidays',
				'normal',
				'default'
			);

			add_meta_box(
				'af_wc_daps_sepcial_day_general_options',
				__( 'Special Days', 'ka-order-delivery-date-and-time-scheduler' ),
				array( $this, 'af_wc_daps_specific_day_general_section_content' ),
				'daps_special_days',
				'normal',
				'default'
			);

			add_meta_box(
				'af_wc_daps_rule_general_options',
				__( 'General Settings', 'ka-order-delivery-date-and-time-scheduler' ),
				array( $this, 'af_wc_daps_rule_general_section_content' ),
				'daps_rules',
				'normal',
				'default'
			);

			add_meta_box(
				'af_wc_daps_rule_advance_options',
				__( 'Advance Settings', 'ka-order-delivery-date-and-time-scheduler' ),
				array( $this, 'af_wc_daps_rule_advance_section_content' ),
				'daps_rules',
				'normal',
				'default'
			);

			add_meta_box(
				'af_wc_daps_time_slot_general_options',
				__( 'Time Slots', 'ka-order-delivery-date-and-time-scheduler' ),
				array( $this, 'af_wc_daps_time_slot_general_section_content' ),
				'daps_rules',
				'normal',
				'default'
			);
		}

		public function af_wc_daps_holidays_general_section_content( $post ) {

			$af_wc_daps_holiday_settings = ! empty( get_post_meta( $post->ID, 'af_wc_daps_holiday_settings', true ) ) ? get_post_meta( $post->ID, 'af_wc_daps_holiday_settings', true ) : array();

			$af_wc_daps_holiday_start_date = isset( $af_wc_daps_holiday_settings['af_wc_daps_holiday_start_date'] ) ? $af_wc_daps_holiday_settings['af_wc_daps_holiday_start_date'] : '';
			$af_wc_daps_holiday_end_date   = isset( $af_wc_daps_holiday_settings['af_wc_daps_holiday_end_date'] ) ? $af_wc_daps_holiday_settings['af_wc_daps_holiday_end_date'] : '';
			$af_wc_daps_repeat_holiday     = isset( $af_wc_daps_holiday_settings['af_wc_daps_repeat_holiday'] ) ? $af_wc_daps_holiday_settings['af_wc_daps_repeat_holiday'] : 'no';
			?>

			<table class="form-table">
				<?php wp_nonce_field( 'af_wc_daps_nonce', 'af_wc_daps_nonce_field' ); ?>

				<tr valign="top">
					<th scope="row"><label><?php esc_html_e( 'Start Date', 'ka-order-delivery-date-and-time-scheduler' ); ?></label></th>
					<td>
						<input type="text" id="af-wc-daps-holiday-start-date" name="af_wc_daps_holiday_start_date" autocomplete="off" required value="<?php echo esc_attr( $af_wc_daps_holiday_start_date ); ?>">
					
						<p class="description"><?php esc_html_e( 'Add the start date of the holiday.', 'ka-order-delivery-date-and-time-scheduler' ); ?></p>
					</td>
				</tr>

				<tr valign="top">
					<th scope="row"><label><?php esc_html_e( 'End Date', 'ka-order-delivery-date-and-time-scheduler' ); ?></label></th>
					<td>
						<input type="text" id="af-wc-daps-holiday-end-date" autocomplete="off" required name="af_wc_daps_holiday_end_date" value="<?php echo esc_attr( $af_wc_daps_holiday_end_date ); ?>">
					
						<p class="description"><?php esc_html_e( 'Add the end date of the holiday.', 'ka-order-delivery-date-and-time-scheduler' ); ?></p>
					</td>
				</tr>

				<tr valign="top">
					<th scope="row"><label><?php esc_html_e( 'Repeat Holiday', 'ka-order-delivery-date-and-time-scheduler' ); ?></label></th>
					<td>
						<input type="checkbox" name="af_wc_daps_repeat_holiday" value="yes" <?php checked( $af_wc_daps_repeat_holiday, 'yes' ); ?>> 
						<p class="description"><?php esc_html_e( 'To set this holiday to repeat on a regular schedule, check this box.', 'ka-order-delivery-date-and-time-scheduler' ); ?></p>
					</td>
					
				</tr>

				
			</table>


			<?php
		}

		public function af_wc_daps_specific_day_general_section_content( $post ) {

			$af_wc_daps_specific_day_settings = ! empty( get_post_meta( $post->ID, 'af_wc_daps_specific_day_settings', true ) ) ? get_post_meta( $post->ID, 'af_wc_daps_specific_day_settings', true ) : array();

			$af_wc_daps_specific_day_date       = isset( $af_wc_daps_specific_day_settings['af_wc_daps_specific_day_date'] ) ? $af_wc_daps_specific_day_settings['af_wc_daps_specific_day_date'] : '';
			$af_wc_daps_specific_day_charges    = isset( $af_wc_daps_specific_day_settings['af_wc_daps_specific_day_charges'] ) ? $af_wc_daps_specific_day_settings['af_wc_daps_specific_day_charges'] : '';
			$af_wc_daps_specific_day_max_orders = isset( $af_wc_daps_specific_day_settings['af_wc_daps_specific_day_max_orders'] ) ? $af_wc_daps_specific_day_settings['af_wc_daps_specific_day_max_orders'] : '';

			?>

			<table class="form-table ">
				<?php wp_nonce_field( 'af_wc_daps_nonce', 'af_wc_daps_nonce_field' ); ?>

				<tr valign="top">
					<th scope="row"><label><?php esc_html_e( 'Date', 'ka-order-delivery-date-and-time-scheduler' ); ?></label></th>
					<td>
						<input type="text" required id="af-wc-daps-specific-day-date" name="af_wc_daps_specific_day_date" autocomplete="off" value="<?php echo esc_attr( $af_wc_daps_specific_day_date ); ?>">
					
						<p class="description"><?php esc_html_e( 'Add the date of specific day.', 'ka-order-delivery-date-and-time-scheduler' ); ?></p>
					</td>
				</tr>

				<tr valign="top">
					<th scope="row"><label><?php esc_html_e( 'Maximum Orders', 'ka-order-delivery-date-and-time-scheduler' ); ?></label></th>
					<td>
						<input type="number" min='0'id="af-wc-daps-specific-day-max-orders" name="af_wc_daps_specific_day_max_orders" value="<?php echo esc_attr( $af_wc_daps_specific_day_max_orders ); ?>">
					
						<p class="description"><?php esc_html_e( 'Add the maximum number of orders allowed on specific day.', 'ka-order-delivery-date-and-time-scheduler' ); ?></p>
					</td>
				</tr>

				<tr valign="top">
					<th scope="row"><label><?php esc_html_e( 'Charges', 'ka-order-delivery-date-and-time-scheduler' ); ?></label></th>
					<td>
						<input type="number" min='1' id="af-wc-daps-specific-day-charges" name="af_wc_daps_specific_day_charges" value="<?php echo esc_attr( $af_wc_daps_specific_day_charges ); ?>">
					
						<p class="description"><?php esc_html_e( 'Add the charges for orders on specific day.', 'ka-order-delivery-date-and-time-scheduler' ); ?></p>
					</td>
				</tr>

				
			</table>


			<?php
		}

		public function af_wc_daps_time_slot_general_section_content( $post ) {
			af_wc_daps_time_slot_section_content($post->ID);
		}

		public function af_wc_daps_product_search() {

			$nonce = isset( $_POST['nonce'] ) && '' !== $_POST['nonce'] ? sanitize_text_field( wp_unslash( $_POST['nonce'] ) ) : '';
			if ( isset( $_POST['nonce'] ) && ! wp_verify_nonce( $nonce, 'af-wc-daps-ajax-nonce' ) ) {
				wp_die( esc_html__( 'Failed security check!', 'ka-order-delivery-date-and-time-scheduler' ) );
			}

			if ( isset( $_POST['q'] ) && '' !== $_POST['q'] ) {
				$query = sanitize_text_field( wp_unslash( $_POST['q'] ) );
			} else {
				$query = '';
			}

			$data_array                             = array();
			$data_array_with_matching_first_word    = array();
			$data_array_without_matching_first_word = array();

			$args = array(
				'post_type'   => array_merge( array( 'product_variation', 'variation' ), array_keys( wc_get_product_types() ) ),
				'post_status' => 'publish',
				'numberposts' => -1,
				's'           => $query,
				'type'        => array( 'simple', 'variation' ),

			);

			$products_query = wc_get_products( $args );

			foreach ( $products_query as $key => $products_obj ) {

				$product_id    = $products_obj->get_id();
				$product_title = $products_obj->get_name();
				$product_type  = $products_obj->get_type();

				if (wc_get_product($product_id)->is_downloadable() || wc_get_product($product_id)->is_virtual()) {
					continue;
				}

				if ( 'variation' == $product_type && ! ( preg_match( '/\s+-\s+/', $product_title ) ) ) {
					continue;
				}

				if ( ! str_contains( strtolower( $product_title ), strtolower( $query ) ) ) {
					continue;
				}

				if ( substr( strtolower( $product_title ), 0, strlen( $query ) ) == substr( strtolower( $query ), 0, strlen( $query ) ) ) {
					$data_array_with_matching_first_word[] = array( $product_id, $product_title );
					continue;
				}

				$data_array_without_matching_first_word[] = array( $product_id, $product_title );
			}

			$data_array = array_merge( $data_array_with_matching_first_word, $data_array_without_matching_first_word );

			echo wp_json_encode( $data_array );
			die();
		}

		public function af_wc_daps_rule_general_section_content( $post ) {

			$af_wc_daps_rule_general_settings = ! empty( get_post_meta( $post->ID, 'af_wc_daps_rule_settings', true ) ) ? get_post_meta( $post->ID, 'af_wc_daps_rule_settings', true ) : array();

			$af_wc_daps_product_ids  = isset( $af_wc_daps_rule_general_settings['af_wc_daps_selected_product_ids'] ) ? $af_wc_daps_rule_general_settings['af_wc_daps_selected_product_ids'] : array();
			$af_wc_daps_category_ids = isset( $af_wc_daps_rule_general_settings['af_wc_daps_selected_category_ids'] ) ? $af_wc_daps_rule_general_settings['af_wc_daps_selected_category_ids'] : array();
			?>
			<table class="form-table af-wc-daps-rule-general-settings-table">
				<?php wp_nonce_field( 'af_wc_daps_nonce', 'af_wc_daps_nonce_field' ); ?>
				<tr valign="top">
					<td scope="row" class="af-wc-daps-rule-label"><?php esc_html_e( 'Products', 'ka-order-delivery-date-and-time-scheduler' ); ?></td>
					<td>
						<select id="af-wc-daps-product-select" name="af_wc_daps_selected_products[]" multiple="multiple">
							<?php
							foreach ( (array) $af_wc_daps_product_ids  as $af_wc_daps_product_id ) {
								if ( $af_wc_daps_product_id ) {
									$af_wc_daps_product = wc_get_product( $af_wc_daps_product_id );
									if ( $af_wc_daps_product ) :
										?>
									<option value="<?php echo esc_attr( $af_wc_daps_product_id ); ?>" selected><?php echo esc_attr( $af_wc_daps_product->get_name() ); ?></option>
										<?php
								endif;
								}
							}
							?>
						</select>
						<p class="description"><?php esc_html_e( 'Select the products on which rule will be applied.', 'ka-order-delivery-date-and-time-scheduler' ); ?></p>
					</td>
				</tr>

				<tr valign="top">
					<td scope="row" class="af-wc-daps-rule-label"><?php esc_html_e( 'Categories', 'ka-order-delivery-date-and-time-scheduler' ); ?></td>
					<td>
						<select id="af-wc-daps-category-select" name="af_wc_daps_selected_categories[]" multiple="multiple">
							<?php
							foreach ( af_wc_daps_get_all_categories() as $af_wc_daps_category_id ) :
								$selected            = in_array( $af_wc_daps_category_id, $af_wc_daps_category_ids ) ? 'selected' : '';
								$af_wc_daps_category = get_term_by( 'id', $af_wc_daps_category_id, 'product_cat' );
								if ( $af_wc_daps_category ) :
									?>
								<option value="<?php echo esc_attr( $af_wc_daps_category_id ); ?>"<?php echo esc_attr( $selected ); ?>><?php echo esc_html( $af_wc_daps_category->name ); ?></option>
									<?php
								endif;
							endforeach;
							?>
						</select>
						<p class="description"><?php esc_html_e( 'Select the categories on which the rule will be applied.', 'ka-order-delivery-date-and-time-scheduler' ); ?></p>
						<p class="description"><b><?php esc_attr_e('Note: ', 'ka-order-delivery-date-and-time-scheduler'); ?></b><?php esc_attr_e('Leave both above fields blank to apply this rule to all products.', 'ka-order-delivery-date-and-time-scheduler'); ?></p>
						</td>	
				</tr>
			</table>

			<?php
		}

		public function af_wc_daps_rule_advance_section_content( $post ) {
			af_wc_daps_rule_advance_settings( $post );
		}

		public function af_wc_daps_save_holidays_content( $post_id ) {
			
			if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
				return;
			}

			if ( ! current_user_can( 'edit_post', $post_id ) ) {
				return;
			}


			$action = filter_input(INPUT_GET, 'action', FILTER_SANITIZE_FULL_SPECIAL_CHARS);

			$exclude_statuses = array( 'auto-draft', 'trash' );
			
			if ( in_array( get_post_status($post_id), $exclude_statuses ) || is_ajax() || 'untrash' === $action ) {
				return;
			}

			$nonce = isset( $_POST['af_wc_daps_nonce_field'] ) ? sanitize_text_field( wp_unslash( $_POST['af_wc_daps_nonce_field'] ) ) : '';

			if ( isset( $_POST['af_wc_daps_nonce_field'] ) && ! wp_verify_nonce( $nonce, 'af_wc_daps_nonce' ) ) {
				wp_die( esc_html__( 'Failed security check!', 'ka-order-delivery-date-and-time-scheduler' ) );
			}

			$af_wc_daps_holiday_start_date = isset($_POST['af_wc_daps_holiday_start_date'])?sanitize_text_field( wp_unslash( $_POST['af_wc_daps_holiday_start_date'] ) ):'';
			$af_wc_daps_holiday_end_date   = isset($_POST['af_wc_daps_holiday_end_date'])?sanitize_text_field( wp_unslash( $_POST['af_wc_daps_holiday_end_date'] ) ):'';
			$af_wc_daps_repeat_holiday     = isset($_POST['af_wc_daps_repeat_holiday'])?sanitize_text_field( wp_unslash( $_POST['af_wc_daps_repeat_holiday'] ) ):'';

			$af_wc_daps_holidays_settings = array(
				'af_wc_daps_holiday_start_date' => $af_wc_daps_holiday_start_date,
				'af_wc_daps_holiday_end_date'   => $af_wc_daps_holiday_end_date,
				'af_wc_daps_repeat_holiday'     => $af_wc_daps_repeat_holiday,
			);

			update_post_meta( $post_id, 'af_wc_daps_holiday_settings', $af_wc_daps_holidays_settings );
		}

		public function af_wc_daps_save_special_days_content( $post_id ) {
			
			if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
				return;
			}

			if ( ! current_user_can( 'edit_post', $post_id ) ) {
				return;
			}


			$action = filter_input(INPUT_GET, 'action', FILTER_SANITIZE_FULL_SPECIAL_CHARS);

			$exclude_statuses = array( 'auto-draft', 'trash' );
			
			if ( in_array( get_post_status($post_id), $exclude_statuses ) || is_ajax() || 'untrash' === $action ) {
				return;
			}

			$nonce = isset( $_POST['af_wc_daps_nonce_field'] ) ? sanitize_text_field( wp_unslash( $_POST['af_wc_daps_nonce_field'] ) ) : '';

			if ( isset( $_POST['af_wc_daps_nonce_field'] ) && ! wp_verify_nonce( $nonce, 'af_wc_daps_nonce' ) ) {
				wp_die( esc_html__( 'Failed security check!', 'ka-order-delivery-date-and-time-scheduler' ) );
			}


			$af_wc_daps_specific_day_date       = isset($_POST['af_wc_daps_specific_day_date'])? sanitize_text_field( wp_unslash( $_POST['af_wc_daps_specific_day_date'] ) ):'';
			$af_wc_daps_specific_day_charges    = isset($_POST['af_wc_daps_specific_day_charges'])? sanitize_text_field( wp_unslash( $_POST['af_wc_daps_specific_day_charges'] ) ):'';
			$af_wc_daps_specific_day_max_orders = isset($_POST['af_wc_daps_specific_day_max_orders']) && ''!= $_POST['af_wc_daps_specific_day_max_orders']? sanitize_text_field( wp_unslash( $_POST['af_wc_daps_specific_day_max_orders'] ) ):'';

			$af_wc_daps_specific_day_settings = array(
				'af_wc_daps_specific_day_date'       => $af_wc_daps_specific_day_date,
				'af_wc_daps_specific_day_charges'    => $af_wc_daps_specific_day_charges,
				'af_wc_daps_specific_day_max_orders' => $af_wc_daps_specific_day_max_orders,
			);

			$af_wc_daps_already_stored_specific_day_settings = (array) get_post_meta( $post_id, 'af_wc_daps_specific_day_settings', true );

			// settings are not saved first time
			if ( ! empty( $af_wc_daps_already_stored_specific_day_settings ) && isset( $af_wc_daps_already_stored_specific_day_settings['af_wc_daps_specific_day_max_orders'] ) ) {
				if ( $af_wc_daps_already_stored_specific_day_settings['af_wc_daps_specific_day_max_orders'] != $af_wc_daps_specific_day_max_orders ) {
		
					$af_wc_daps_remaining_orders     = get_post_meta( $post_id, 'af_wc_daps_remaining_orders_of_specific_day', true );
					$af_wc_daps_new_remaining_orders = intval( $af_wc_daps_remaining_orders ) - ( intval( $af_wc_daps_already_stored_specific_day_settings['af_wc_daps_specific_day_max_orders'] ) - intval( $af_wc_daps_specific_day_max_orders ) );

					if ('' == $af_wc_daps_specific_day_max_orders) {
						update_post_meta( $post_id, 'af_wc_daps_remaining_orders_of_specific_day', '' );
					} elseif ( $af_wc_daps_new_remaining_orders < 0 ) {
						update_post_meta( $post_id, 'af_wc_daps_remaining_orders_of_specific_day', 0 );
					} else {
						update_post_meta( $post_id, 'af_wc_daps_remaining_orders_of_specific_day', $af_wc_daps_new_remaining_orders );
					}
				}
			} else {
				update_post_meta( $post_id, 'af_wc_daps_remaining_orders_of_specific_day', $af_wc_daps_specific_day_max_orders );
			}

			update_post_meta( $post_id, 'af_wc_daps_specific_day_settings', $af_wc_daps_specific_day_settings );
		}

		public function af_wc_daps_save_metabox_content( $post_id ) {
			
			if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
				return;
			}

			if ( ! current_user_can( 'edit_post', $post_id ) ) {
				return;
			}

			$action = filter_input(INPUT_GET, 'action', FILTER_SANITIZE_FULL_SPECIAL_CHARS);

			$exclude_statuses = array( 'auto-draft', 'trash' );
			
			if ( in_array( get_post_status($post_id), $exclude_statuses ) || is_ajax() || 'untrash' === $action ) {
				return;
			}

			$nonce = isset( $_POST['af_wc_daps_nonce_field'] ) ? sanitize_text_field( wp_unslash( $_POST['af_wc_daps_nonce_field'] ) ) : '';

			if ( isset( $_POST['af_wc_daps_nonce_field'] ) && ! wp_verify_nonce( $nonce, 'af_wc_daps_nonce' ) ) {
				wp_die( esc_html__( 'Failed security check!', 'ka-order-delivery-date-and-time-scheduler' ) );
			}

			
			//time slot settings
			if (isset($_POST['af_wc_daps_time_slot_settings'])) {
				$af_wc_daps_row_counter        = isset($_POST['row_counter'])?sanitize_text_field(wp_unslash($_POST['row_counter'])):'0';
				$af_wc_daps_time_slot_settings = sanitize_meta('', wp_unslash($_POST['af_wc_daps_time_slot_settings']), '');

				$already_stored_timeslot_settings = get_post_meta($post_id, 'af_wc_daps_time_slot_settings', true);

				foreach ($af_wc_daps_time_slot_settings as $actual_key=> $time_slot) {
					$orders_per_slot = isset($time_slot['orders_per_slot'])? $time_slot['orders_per_slot']:'';

					$old_orders_per_slot = isset($already_stored_timeslot_settings[ $actual_key ]['orders_per_slot']) && '' != $already_stored_timeslot_settings[ $actual_key ]['orders_per_slot'] ?$already_stored_timeslot_settings[ $actual_key ]['orders_per_slot']:0;

					$af_wc_daps_time_slot_per_order_details = get_post_meta($post_id, 'af_wc_daps_time_slot_per_order_details', true);

					if (is_array($af_wc_daps_time_slot_per_order_details) && !empty($af_wc_daps_time_slot_per_order_details)) {
						foreach ($af_wc_daps_time_slot_per_order_details as $key =>  $value ) {

							if (preg_match('/[^_]+$/', $key, $matches)) {
								$timeslot_key = $matches[0];

								if ($actual_key == $timeslot_key ) {
									
									if ( '' == $orders_per_slot) {
										unset($af_wc_daps_time_slot_per_order_details[ $key ]);
									} elseif ($old_orders_per_slot != $orders_per_slot) { 
										$new_value                                      = $value + $orders_per_slot - $old_orders_per_slot;
										$af_wc_daps_time_slot_per_order_details[ $key ] = $new_value>0?$new_value:0;
									}
								}
							}
						}

						update_post_meta($post_id, 'af_wc_daps_time_slot_per_order_details', $af_wc_daps_time_slot_per_order_details);
					}
				}
				update_post_meta($post_id, 'af_wc_daps_row_counter', $af_wc_daps_row_counter);
				update_post_meta( $post_id, 'af_wc_daps_time_slot_settings', $af_wc_daps_time_slot_settings );
			} else {
				update_post_meta( $post_id, 'af_wc_daps_time_slot_settings', array() );
				update_post_meta($post_id, 'af_wc_daps_row_counter', '');
			}
					
			if ( isset( $_POST['af_wc_daps_enable_rule'] ) || isset( $_POST['af_wc_daps_rule_delivery_selection_mode'] ) || isset( $_POST['af_wc_daps_rule_minimum_delivery_days'] ) || isset( $_POST['af_wc_daps_rule_maximum_delivery_days'] ) || isset( $_POST['af_wc_daps_selected_products'] ) || isset( $_POST['af_wc_daps_selected_categories'] ) || isset( $_POST['af_wc_daps_rule_available_days'] ) || isset( $_POST['af_wc_daps_rule_available_days_of_week'] ) || isset( $_POST['af_wc_daps_rule_time_selector_start_time'] ) || isset( $_POST['af_wc_daps_rule_time_selector_end_time'] ) || isset( $_POST['af_wc_daps_rule_week_day_fee'] ) || isset( $_POST['af_wc_daps_rule_min_available_days'] ) || isset( $_POST['af_wc_daps_rule_max_available_days'] ) ) {

				$af_wc_daps_enable_rule = isset($_POST['af_wc_daps_enable_rule'])?sanitize_text_field( wp_unslash( $_POST['af_wc_daps_enable_rule'] ) ): '';

				$af_wc_daps_rule_delivery_selection_mode = isset($_POST['af_wc_daps_rule_delivery_selection_mode'])?sanitize_text_field( wp_unslash( $_POST['af_wc_daps_rule_delivery_selection_mode'] ) ):'';

				$af_wc_daps_rule_minimum_delivery_days = isset($_POST['af_wc_daps_rule_minimum_delivery_days'])?sanitize_text_field( wp_unslash( $_POST['af_wc_daps_rule_minimum_delivery_days'] ) ):'';

				$af_wc_daps_rule_maximum_delivery_days = isset($_POST['af_wc_daps_rule_maximum_delivery_days'])?sanitize_text_field( wp_unslash( $_POST['af_wc_daps_rule_maximum_delivery_days'] ) ):'';

				$af_wc_daps_rule_expected_date_message_text = isset($_POST['af_wc_daps_rule_expected_date_message_text']) ?sanitize_text_field( wp_unslash( $_POST['af_wc_daps_rule_expected_date_message_text'] ) ):'';

				$af_wc_daps_rule_min_available_days = isset($_POST['af_wc_daps_rule_min_available_days'])?sanitize_text_field( wp_unslash( $_POST['af_wc_daps_rule_min_available_days'] ) ):'';

				$af_wc_daps_rule_max_available_days = isset($_POST['af_wc_daps_rule_max_available_days'])?sanitize_text_field( wp_unslash( $_POST['af_wc_daps_rule_max_available_days'] ) ):'';


				if (isset($_POST['af_wc_daps_selected_products'])) {
					$af_wc_daps_selected_product_ids = sanitize_meta('', wp_unslash($_POST['af_wc_daps_selected_products']), '');
				} else {
					$af_wc_daps_selected_product_ids = array();
				}

				if (isset($_POST['af_wc_daps_selected_categories'])) {
					$af_wc_daps_selected_category_ids = sanitize_meta('', wp_unslash($_POST['af_wc_daps_selected_categories']), '');
				} else {
					$af_wc_daps_selected_category_ids = array();
				}

				$af_wc_daps_rule_available_days_of_week = sanitize_meta( '', wp_unslash( isset( $_POST['af_wc_daps_rule_available_days_of_week'] ) ? $_POST['af_wc_daps_rule_available_days_of_week'] : array() ), '' );

				$af_wc_daps_rule_time_selection_type = isset($_POST['af_wc_daps_rule_time_selection_type'])?sanitize_text_field( wp_unslash( $_POST['af_wc_daps_rule_time_selection_type'] ) ):'';

				$af_wc_daps_rule_time_selector_start_time    = isset($_POST['af_wc_daps_rule_time_selector_start_time'])?sanitize_text_field( wp_unslash( $_POST['af_wc_daps_rule_time_selector_start_time'] ) ):'';
				$af_wc_daps_rule_time_selector_end_time      = isset($_POST['af_wc_daps_rule_time_selector_end_time'])?sanitize_text_field( wp_unslash( $_POST['af_wc_daps_rule_time_selector_end_time'] ) ):'';
				$af_wc_daps_rule_time_selector_slot_duration = isset($_POST['af_wc_daps_rule_time_selector_slot_duration'])?sanitize_text_field( wp_unslash( $_POST['af_wc_daps_rule_time_selector_slot_duration'] ) ):'';
				$orders_per_slot                             = isset($_POST['orders_per_slot'])?sanitize_text_field( wp_unslash( $_POST['orders_per_slot'] ) ):'';

				//handling the orders per slot 
				$already_stored_rule_settings = get_post_meta($post_id, 'af_wc_daps_rule_settings', true);

				if (is_array($already_stored_rule_settings) && !empty($already_stored_rule_settings)) {
					
					$current_time_selector_slot_duration = '' != $af_wc_daps_rule_time_selector_slot_duration ? $af_wc_daps_rule_time_selector_slot_duration:'30';
					$start_time                          = isset($already_stored_rule_settings['af_wc_daps_rule_time_selector_start_time'])?$already_stored_rule_settings['af_wc_daps_rule_time_selector_start_time']:'';
					$end_time                            = isset($already_stored_rule_settings['af_wc_daps_rule_time_selector_end_time'])?$already_stored_rule_settings['af_wc_daps_rule_time_selector_end_time']:'';
					$slot_duration                       = isset($already_stored_rule_settings['af_wc_daps_rule_time_selector_slot_duration']) && '' != $already_stored_rule_settings['af_wc_daps_rule_time_selector_slot_duration'] ?$already_stored_rule_settings['af_wc_daps_rule_time_selector_slot_duration']:'30';

					$old_orders_per_slot = isset($already_stored_rule_settings['orders_per_slot']) && '' != $already_stored_rule_settings['orders_per_slot'] ?$already_stored_rule_settings['orders_per_slot']:0;

					//if any of these values changes then reset the slot data and per order count
					if ($start_time != $af_wc_daps_rule_time_selector_start_time || $end_time != $af_wc_daps_rule_time_selector_end_time || ( $slot_duration != $current_time_selector_slot_duration )) {
						update_post_meta($post_id, 'af_wc_daps_time_select_slot_per_order_details', array());
					}


					if ('' == $orders_per_slot) {
						update_post_meta($post_id, 'af_wc_daps_time_select_slot_per_order_details', array());

					} elseif ($old_orders_per_slot != $orders_per_slot) { 
						$af_wc_daps_time_select_slot_per_order_details = get_post_meta($post_id, 'af_wc_daps_time_select_slot_per_order_details', true);

						if (is_array($af_wc_daps_time_select_slot_per_order_details) && !empty($af_wc_daps_time_select_slot_per_order_details)) {
							foreach ($af_wc_daps_time_select_slot_per_order_details as $key =>  $value ) {
								$new_value = $value + $orders_per_slot - $old_orders_per_slot;
								$af_wc_daps_time_select_slot_per_order_details[ $key ] = $new_value < 0 ? 0 : $new_value;
							}

							update_post_meta($post_id, 'af_wc_daps_time_select_slot_per_order_details', $af_wc_daps_time_select_slot_per_order_details);
						}
					} 
				}
				
				$af_wc_daps_rule_week_day_fee = sanitize_meta( '', wp_unslash( isset( $_POST['af_wc_daps_rule_week_day_fee'] ) ? $_POST['af_wc_daps_rule_week_day_fee'] : array() ), '' );

				if ( get_post_type( $post_id ) == 'product' ) {
					$af_wc_daps_selected_product_ids[] = $post_id;
				}

				$af_wc_daps_rule_settings = array(
					'af_wc_daps_enable_rule'              => $af_wc_daps_enable_rule,
					'af_wc_daps_selected_product_ids'     => $af_wc_daps_selected_product_ids,
					'af_wc_daps_product_ids_for_rule'     => $af_wc_daps_selected_product_ids,
					'af_wc_daps_selected_category_ids'    => $af_wc_daps_selected_category_ids,
					'af_wc_daps_rule_delivery_selection_mode' => $af_wc_daps_rule_delivery_selection_mode,
					'af_wc_daps_rule_minimum_delivery_days' => $af_wc_daps_rule_minimum_delivery_days,
					'af_wc_daps_rule_maximum_delivery_days' => $af_wc_daps_rule_maximum_delivery_days,
					'af_wc_daps_rule_expected_date_message_text' => $af_wc_daps_rule_expected_date_message_text,
					'af_wc_daps_rule_min_available_days'  => $af_wc_daps_rule_min_available_days,
					'af_wc_daps_rule_max_available_days'  => $af_wc_daps_rule_max_available_days,
					'af_wc_daps_rule_available_days_of_week' => $af_wc_daps_rule_available_days_of_week,
					'af_wc_daps_rule_time_selection_type' => $af_wc_daps_rule_time_selection_type,
					'af_wc_daps_rule_time_selector_start_time' => $af_wc_daps_rule_time_selector_start_time,
					'af_wc_daps_rule_time_selector_end_time' => $af_wc_daps_rule_time_selector_end_time,
					'af_wc_daps_rule_time_selector_slot_duration' => $af_wc_daps_rule_time_selector_slot_duration,
					'orders_per_slot'                     => $orders_per_slot,
					'af_wc_daps_rule_week_day_fee'        => $af_wc_daps_rule_week_day_fee,
				);

				update_post_meta( $post_id, 'af_wc_daps_rule_settings', $af_wc_daps_rule_settings );
			}
		}


		public function af_wc_daps_display_tabs() {
			global $post, $typenow;
			$screen = get_current_screen();

			if ( $screen && in_array( $screen->id, $this->af_wc_daps_get_tab_screen_ids(), true ) ) {

				$tabs = array(
					'daps_rules'        => array(
						'title' => __( 'Rules', 'ka-order-delivery-date-and-time-scheduler' ),
						'url'   => admin_url( 'edit.php?post_type=daps_rules' ),
					),
					'daps_holidays'     => array(
						'title' => __( 'Holidays', 'ka-order-delivery-date-and-time-scheduler' ),
						'url'   => admin_url( 'edit.php?post_type=daps_holidays' ),
					),
					'daps_special_days' => array(
						'title' => __( 'Special Days', 'ka-order-delivery-date-and-time-scheduler' ),
						'url'   => admin_url( 'edit.php?post_type=daps_special_days' ),
					),
					'order_deliveries'  => array(
						'title' => __( 'Order Deliveries', 'ka-order-delivery-date-and-time-scheduler' ),
						'url'   => admin_url( 'edit.php?post_type=daps_order_delivery' ),
					),
				);

				if ( is_array( $tabs ) ) {
					?>
					<div class="wrap woocommerce">
						<h2 class="nav-tab-wrapper woo-nav-tab-wrapper">
							<?php
							$current_tab = $this->af_wc_daps_get_current_tab();
							foreach ( $tabs as $id => $tab_data ) {
								$class = $id === $current_tab ? array( 'nav-tab', 'nav-tab-active' ) : array( 'nav-tab' );
								printf( '<a href="%1$s" class="%2$s">%3$s</a>', esc_url( $tab_data['url'] ), implode( ' ', array_map( 'sanitize_html_class', $class ) ), esc_html( $tab_data['title'] ) );
							}
							?>
						</h2>
					</div>
					<?php
				}
			}
		}

		public function af_wc_daps_get_tab_screen_ids() {
			$tabs_screens = array(
				'daps_holidays',
				'edit-daps_holidays',
				'daps_special_days',
				'edit-daps_special_days',
				'daps_rules',
				'edit-daps_rules',
				'daps_order_delivery',
				'edit-daps_order_delivery',
			);
			return $tabs_screens;
		}

		public function af_wc_daps_get_current_tab() {
			$screen = get_current_screen();

			switch ( $screen->id ) {
				case 'daps_holidays':
				case 'edit-daps_holidays':
					return 'daps_holidays';
				case 'daps_special_days':
				case 'edit-daps_special_days':
					return 'daps_special_days';
				case 'daps_rules':
				case 'edit-daps_rules':
					return 'daps_rules';
				case 'daps_order_delivery':
				case 'edit-daps_order_delivery':
					return 'order_deliveries';
			}
		}

		public function af_wc_daps_remove_submenu_link() {
			
			global $pagenow, $typenow;

			$current_post_id = filter_input(INPUT_GET, 'post', FILTER_VALIDATE_INT);
			$current_page    = filter_input(INPUT_GET, 'page', FILTER_SANITIZE_FULL_SPECIAL_CHARS);

			if ( ( 'edit.php' === $pagenow && 'daps_holidays' === $typenow )
				|| ( 'post.php' === $pagenow && isset( $current_post_id ) && 'daps_holidays' === get_post_type( sanitize_text_field( $current_post_id ) ) )
			) {
				remove_submenu_page( 'woocommerce', 'edit.php?post_type=daps_special_days' );
				remove_submenu_page( 'woocommerce', 'edit.php?post_type=daps_rules' );
				remove_submenu_page( 'woocommerce', 'edit.php?post_type=daps_order_delivery' );

			} elseif ( ( 'edit.php' === $pagenow && 'daps_special_days' === $typenow )
				|| ( 'post.php' === $pagenow && isset( $current_post_id ) && 'daps_special_days' === get_post_type( sanitize_text_field( $current_post_id ) ) )
			) {
				remove_submenu_page( 'woocommerce', 'edit.php?post_type=daps_holidays' );
				remove_submenu_page( 'woocommerce', 'edit.php?post_type=daps_rules' );
				remove_submenu_page( 'woocommerce', 'edit.php?post_type=daps_order_delivery' );

			} elseif ( ( 'edit.php' === $pagenow && 'daps_rules' === $typenow )
				|| ( 'post.php' === $pagenow && isset( $current_post_id ) && 'daps_rules' === get_post_type( sanitize_text_field( $current_post_id ) ) )
			) {
				remove_submenu_page( 'woocommerce', 'edit.php?post_type=daps_special_days' );
				remove_submenu_page( 'woocommerce', 'edit.php?post_type=daps_holidays' );
				remove_submenu_page( 'woocommerce', 'edit.php?post_type=daps_order_delivery' );
			} elseif ( ( 'edit.php' === $pagenow && 'daps_order_delivery' === $typenow )
				|| ( 'post.php' === $pagenow && isset( $current_post_id ) && 'daps_order_delivery' === get_post_type( sanitize_text_field( $current_post_id ) ) )
			) {
				remove_submenu_page( 'woocommerce', 'edit.php?post_type=daps_special_days' );
				remove_submenu_page( 'woocommerce', 'edit.php?post_type=daps_holidays' );
				remove_submenu_page( 'woocommerce', 'edit.php?post_type=daps_rules' );

			} else {
				remove_submenu_page( 'woocommerce', 'edit.php?post_type=daps_special_days' );
				remove_submenu_page( 'woocommerce', 'edit.php?post_type=daps_holidays' );
				remove_submenu_page( 'woocommerce', 'edit.php?post_type=daps_order_delivery' );
				remove_submenu_page( 'woocommerce', 'edit.php?post_type=daps_rules' );

				add_submenu_page(
					'woocommerce', 
					__('Delivery Date & Time', 'ka-order-delivery-date-and-time-scheduler'), 
					__('Delivery Date & Time', 'ka-order-delivery-date-and-time-scheduler'), 
					'manage_woocommerce', 
					'edit.php?post_type=daps_rules',
					array(),
					3
				);
			
				
				
			}
		}

		public function af_wc_daps_display_delivery_info_for_individual_product( $item_id, $item, $product ) {
			$af_wc_daps_delivery_info_data = (array) wc_get_order_item_meta( $item_id, 'af_wc_daps_delivery_info_data', true );
			$af_wc_daps_expected_date_data = (array) wc_get_order_item_meta( $item_id, 'af_wc_daps_expected_date_data', true );

			if ( ! empty( $af_wc_daps_delivery_info_data ) ) {

				foreach ( $af_wc_daps_delivery_info_data as $key => $value ) {

					$key = af_wc_daps_formatted_key_names( $key );

					if ('af_wc_daps_single_product_page_enable' == $key || 'applied_time_slot_details' == $key) {
						continue;
					}

					if ( ( 'Total Fee' == $key || 'Day Fee' == $key ) && 0 == $value ) {
						continue;
					}
					if ( '' != $value ) {
						if ('Total Fee' == $key || 'Day Fee' == $key) {
							echo '<br>';
							echo '<b>' . esc_attr( $key ) . ' : </b>' . esc_attr( get_woocommerce_currency_symbol() ) . esc_attr( $value );
							continue;
						}
						echo '<br>';
						echo '<b>' . esc_attr( $key ) . ' :</b> ' . esc_attr( $value );
					}
				}
			}

			if ( ! empty( $af_wc_daps_expected_date_data ) ) {
				foreach ( $af_wc_daps_expected_date_data as $key => $value ) {
					if ( '' == $value ) {
						continue;
					}
					echo '<br>';
					echo '<b>' . esc_html__('Expected Date :', 'ka-order-delivery-date-and-time-scheduler') . ' </b>' . esc_attr( $value );
				}
			}
		}


		public function af_wc_daps_display_delivery_info_data_at_edit_order_page( $order_id ) {
			$order =wc_get_order($order_id);

			$af_wc_daps_order_data                       = (array) $order->get_meta('af_wc_daps_order_data_checkout_page');
			$af_wc_daps_expected_date_data_checkout_page = (array) $order->get_meta('af_wc_daps_expected_date_data_checkout_page');

			if ( ! empty( $af_wc_daps_order_data ) ) {

				echo '<br>';
				echo "<table class='af-wc-daps-delivery-info-table-order-edit-page' style=float:right> ";

				foreach ( $af_wc_daps_order_data as $key => $value ) {

					$key = af_wc_daps_formatted_key_names( $key );
					
					if ( ( ( 'Total Fee' == $key || 'Day Fee' == $key ) && 0 == $value ) || 'applied_time_slot_details' == $key || 'applied_time_slot_details_checkout_page' == $key

) {
						continue;
					}
					if ( '' != $value ) {
						$af_wc_daps_rule_applied_on_this_order = true;
						if ('af_wc_daps_enable_checkout_page' == $key || 'af_wc_daps_single_product_page_enable' == $key ) {
							continue;
						}
						if ( 'Total Fee' == $key || 'Day Fee' == $key ) {
							echo '<tr>';
							echo '<td>';
							echo '<label>' . esc_attr($key) . ':</label>';
							echo '</td>';
							echo "<td style='width:1%'></td>";
							echo '<td>';
							echo '<b>' . esc_attr(get_woocommerce_currency_symbol() . $value) . '</b>';
							echo '</td>';
							echo '</tr>';
							continue;
						}
							echo '<tr>';
							echo '<td>';
							echo '<label>' . esc_attr( $key ) . ':</label>';
							echo '</td>';
							echo "<td style='width:1%'></td>";
							echo '<td>';
							echo '<b>' . esc_attr( $value ) . '</b>';
							echo '</td>';
							echo '</tr>';
					}
				}
				echo '</table>';
			}

			if ( ! empty( $af_wc_daps_expected_date_data_checkout_page ) ) {
				foreach ( $af_wc_daps_expected_date_data_checkout_page as $key => $value ) {
					if ( '' != $value ) {
						$af_wc_daps_rule_applied_on_this_order = true;
						echo '<br>';
						echo '<b>' . esc_html__('Expected Date :', 'ka-order-delivery-date-and-time-scheduler') . '</b>' . esc_attr( $value );
					}
				}
			}
		}
	}
	new Af_Wc_Daps_Admin();
}


